Package: test.assessr
Title: Assessing Package Test Reliability and Quality
Version: 2.1.2
Authors@R: c(
    person("Edward", "Gillian", , "edward.gillian-ext@sanofi.com", role = c("cre", "aut"), comment = c(ORCID = "0000-0003-2732-5107")),
    person("Hugo", "Bottois", , "hugo.bottois@sanofi.com", role = "aut", comment = c(ORCID = "0000-0003-4674-0875")),
    person("Paulin", "Charliquart", , "paulin.charliquart@sanofi.com", role = "aut"),
    person("Andre", "Couturier", , "andre.couturier@sanofi.com", role = "aut"),
    person("Sanofi", role = c("cph", "fnd")))
Description: A reliable and validated tool that calculates unit test coverage for R packages with standard testing frameworks and non-standard testing frameworks.
License: GPL (>= 2)
BugReports: https://github.com/pharmaverse/test.assessr/issues
Depends: R (>= 4.1.0)
Imports: callr, checkmate, covr (>= 3.6.0), dplyr, fs, jsonlite,
        pkgload, remotes, rlang, rmarkdown, RUnit, stringr (>= 1.5.0),
        testthat (>= 3.0.0), tinytest, tidyr, utils, withr
Suggests: curl, devtools, DT, here, kableExtra (>= 1.4.0), knitr,
        methods, R6, S7, roxygen2, testit (>= 0.13), tidyselect, tools,
        mockery
Encoding: UTF-8
Config/testthat/edition: 3
Config/build/clean-inst-doc: false
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-06-10 05:31:57 UTC; u1004798
Author: Edward Gillian [cre, aut] (ORCID:
    <https://orcid.org/0000-0003-2732-5107>),
  Hugo Bottois [aut] (ORCID: <https://orcid.org/0000-0003-4674-0875>),
  Paulin Charliquart [aut],
  Andre Couturier [aut],
  Sanofi [cph, fnd]
Maintainer: Edward Gillian <edward.gillian-ext@sanofi.com>
Repository: CRAN
Date/Publication: 2026-06-10 09:10:02 UTC
Built: R 4.6.0; ; 2026-06-10 11:29:29 UTC; unix
