# Generated by roxygen2: do not edit by hand

export(extract_short_path)
export(generate_test_report)
export(get_package_coverage)
export(get_pkg_name)
export(install_package_local)
export(run_coverage)
export(run_covr_modes)
export(set_up_pkg)
importFrom(covr,environment_coverage)
importFrom(covr,package_coverage)
importFrom(dplyr,"%>%")
importFrom(fs,dir_exists)
importFrom(fs,path_abs)
importFrom(rlang,"%||%")
importFrom(rmarkdown,render)
importFrom(stats,setNames)
importFrom(testthat,test_dir)
importFrom(utils,capture.output)
importFrom(utils,data)
importFrom(utils,getParseData)
importFrom(utils,head)
importFrom(utils,read.csv)
importFrom(utils,read.table)
importFrom(utils,tail)
