globalVariables(c(
                  ".data",
                  "bioc_unit_tests_dir_path",
                  "Code_script",
                  "code_script",
                  "cov_env",
                  "Freq",
                  "Test_Coverage",
                  "Var1",
                  "coverage_percent",
                  "covered",
                  "comments",
                  "dependencies",
                  "description",
                  "descriptions",
                  "documentation",
                  "error",
                  "exported_function",
                  "exported_functions",
                  "filename",
                  "functions",
                  "function_body",
                  "function_calls",
                  "function_type",
                  "has_bug_reports_url",
                  "imp_count",
                  "is_assignment",
                  "is_parameter",
                  "join_key",
                  "link_count",
                  "matched",
                  "name",
                  "overall_risk_score",
                  "package",
                  "package_name",
                  "pkg_name",
                  "pkg_version",
                  "server",
                  "status",
                  "sug_count",
                  "suggested_function",
                  "targeted_package",
                  "test_file",
                  "testthat_summary",
                  "type",
                  "value",
                  "Date", 
                  "Month", 
                  "Nb_of_distinct_IPs",
                  "Nb_of_downloads",
                  "Year",
                  "count",
                  "cumulative_downloads",
                  "day",
                  "desc",
                  "downloads",
                  "month",
                  "n_commits",
                  "week_start",
                  "yarn",
                  "year"
                  ))
