% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_covr_modes.R
\name{detect_frameworks}
\alias{detect_frameworks}
\title{Detect which test frameworks are present in a package}
\usage{
detect_frameworks(test_pkg_data)
}
\arguments{
\item{test_pkg_data}{List from \code{check_pkg_tests_and_snaps}.}
}
\value{
Character vector of framework IDs: "testthat", "testit", "tests_base",
  "tinytest", "data.table", "bioconductor".
}
\description{
Detect which test frameworks are present in a package
}
\keyword{internal}
