% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generate_test_report.R
\name{convert_vectors_to_df}
\alias{convert_vectors_to_df}
\title{This internal function takes named character vectors (e.g., `tests_skipped`, `tests_passing`)
from a list and converts them into data frames for easier reporting or processing.}
\usage{
convert_vectors_to_df(x, elems)
}
\arguments{
\item{x}{A list containing character vectors (e.g., `test_results`).}

\item{elems}{A character vector of element names to convert (must exist in `x`).}
}
\value{
A named list of data frames, where each data frame has one column:
\describe{
  \item{file}{The file names or paths from the original character vector.}
}
}
\description{
This internal function takes named character vectors (e.g., `tests_skipped`, `tests_passing`)
from a list and converts them into data frames for easier reporting or processing.
}
\keyword{internal}
