% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_covr_skip_stf.R
\name{create_fail_reporter}
\alias{create_fail_reporter}
\title{Create Fail Reporter}
\usage{
create_fail_reporter(test_path, pkg, cov_env)
}
\arguments{
\item{test_path}{Character string specifying the path to the test directory.}

\item{pkg}{Character string specifying the package name.}

\item{cov_env}{environment convr environment.}
}
\value{
A list of test results with corrected contexts, or `NULL` if an error occurs.
}
\description{
Executes all tests in a given directory using `testthat::test_dir()`
and applies context fixes to ensure each test has a valid context.
}
\keyword{internal}
