% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_covr_skip_runit_nstf.R
\name{prepare_for_bioc_runit}
\alias{prepare_for_bioc_runit}
\title{Prepare a BioC/RUnit package for testing
Detaches common BioC infrastructure packages (if attached) and load_all() the target
package before RUnit tests. Also clears R_TESTS to avoid interference from the check harness.}
\usage{
prepare_for_bioc_runit(pkg_name, pkg_source_path, detach_bioc_infra)
}
\arguments{
\item{pkg_name}{Character. Package name.}

\item{pkg_source_path}{Character. Path to package source.}

\item{detach_bioc_infra}{Logical. If TRUE (default for BioC packages), detach
IRanges, S4Vectors, BiocGenerics. Set to FALSE for RUnit-only packages.}
}
\description{
Prepare a BioC/RUnit package for testing
Detaches common BioC infrastructure packages (if attached) and load_all() the target
package before RUnit tests. Also clears R_TESTS to avoid interference from the check harness.
}
\keyword{internal}
