% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_covr_skip_bioc_nstf.R,
%   R/run_covr_skip_runit_nstf.R
\name{remove_run_unitTests_mapping}
\alias{remove_run_unitTests_mapping}
\title{Remove mapping rows that point to run_unitTests.R}
\usage{
remove_run_unitTests_mapping(mapping, test_pkg_data)

remove_run_unitTests_mapping(mapping, test_pkg_data)
}
\arguments{
\item{mapping}{A data.frame with columns `source_file` and `test_file`
returned by get_source_test_mapping_nstf().}

\item{test_pkg_data}{A list or environment containing `bioc_run_ut_path`,
e.g., "/tmp/.../BiocGenerics/tests/run_unitTests.R".}
}
\value{
The filtered mapping data.frame with run_unitTests.R rows removed.

The filtered mapping data.frame with run_unitTests.R rows removed.
}
\description{
Remove mapping rows that point to run_unitTests.R

Remove mapping rows that point to run_unitTests.R
}
\keyword{internal}
