% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_covr_skip_runit_nstf.R
\name{is_bioc_runit_layout}
\alias{is_bioc_runit_layout}
\title{Detect if a package uses the Bioconductor RUnit layout}
\usage{
is_bioc_runit_layout(pkg_source_path, test_pkg_data = NULL)
}
\description{
Heuristics:
 - inst/unitTests exists, OR
 - tests/run_unitTests.R exists, OR
 - explicit flag in test_pkg_data$has_BioG_test
}
\keyword{internal}
