% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_covr_skip_runit_nstf.R
\name{detach_pkg_if_attached}
\alias{detach_pkg_if_attached}
\title{Detach a package if attached (internal)}
\usage{
detach_pkg_if_attached(pkg, unload_namespace = TRUE, quiet = TRUE)
}
\description{
Safely detaches a package from the search path and optionally unloads its namespace.
}
\keyword{internal}
