% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_covr_skip_dt_nstf.R
\name{prepare_tests_for_package_dt}
\alias{prepare_tests_for_package_dt}
\title{Prepare package-specific test environment for data.table (internal)}
\usage{
prepare_tests_for_package_dt(
  pkg_name,
  pkg_source_path,
  test_files,
  mode = c("mirror")
)
}
\description{
For data.table, Only "mirror" mode is supported here: 
copy inst/tests/* -> tests/ and clean up.
}
\details{
Returns a list with:
  - test_files: character() of wrapper files to execute
  - teardown:   function() to revert any changes (delete mirrored files, unset env)
}
\keyword{internal}
