% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_covr_skip_nstf.R
\name{check_covr_skip_testit}
\alias{check_covr_skip_testit}
\title{Run testit tests individually, capturing all warnings and errors, with dynamic package root detection}
\usage{
check_covr_skip_testit(pkg_name, mapping, test_path)
}
\arguments{
\item{pkg_name}{The package name (not used in this function, but kept for compatibility)}

\item{mapping}{A data frame with a column `test_file` listing test file names}

\item{test_path}{The path to the test directory (e.g., tests/testit)}
}
\value{
A data frame with test_file and issue_type columns
}
\description{
Run testit tests individually, capturing all warnings and errors, with dynamic package root detection
}
\seealso{
Other nstf_utility: 
\code{\link{create_base_tests_coverage}()},
\code{\link{create_nstf_covr_list}()},
\code{\link{create_tinytest_coverage}()},
\code{\link{get_nstf_test_path}()},
\code{\link{get_source_test_mapping_nstf}()},
\code{\link{restrict_test_paths_to_framework}()},
\code{\link{run_covr_skip_nstf}()}
}
\concept{nstf_utility}
\keyword{internal}
